/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// Object maps
//========================================================================================================
var EPComboBoxMap = new Array();

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at document onclick
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epEPComboBoxDocumentOnClickHandler(event) {
	var targetHtmlE = null;
	if (browser.isIE)
		targetHtmlE = window.event.srcElement;
	else
		targetHtmlE = event.target;

	for (x in EPComboBoxMap) {
		var combo = EPComboBoxMap[x];
		if (combo.toggleE && combo.toggleE.htmlE != targetHtmlE)
			combo.hideListBox();
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Object to hold a value and rendering style class
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function EPComboBoxItem(itemValue, itemClass) {
	this.itemValue = itemValue;
	this.itemClass = itemClass;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Our EPComboBox object.  2 stage construction
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function EPComboBox(htmlId, itemNormalClass, actionOnEnter, actionOnSelection, textMatching, setWidth) {
	this.htmlId = htmlId;
	this.itemNormalClass = itemNormalClass;
	this.actionOnEnter = actionOnEnter;
	this.actionOnSelection = actionOnSelection;
	this.textMatching  = textMatching;
	this.setWidth = setWidth;
	
	this.textFieldE = null;
	this.toggleE = null;
	this.listBoxE = null;

	this.items = new Array();

	EPComboBoxMap[htmlId] = this;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Add an item to the list of possible options
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.addItem = function(itemValue, itemClass) {
	var newItemClass = itemClass;
	if (itemClass == undefined)
		newItemClass = this.itemNormalClass;
	var item = new EPComboBoxItem(itemValue,newItemClass);
	this.items[this.items.length] = item;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Hides the popup list
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.hideListBox = function() {
	this.listBoxE.setVisible(false);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Shows the popup list
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.showListBox = function() {
	this.listBoxE.setVisible(true);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Resizes the popup list next to the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.adjustSize = function() {
	var x = this.textFieldE.getPageX();
	var y = this.textFieldE.getPageY() + this.textFieldE.getHeight();
	
	this.listBoxE.setX(x);
	this.listBoxE.setY(y);
	if (this.setWidth) {
		var tw = this.textFieldE.getWidth() + this.toggleE.getWidth();
		if (this.listBoxE.getWidth() < tw)
			this.listBoxE.setWidth(tw);
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to set the focus to the textfield
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.setTextFieldFocus = function() {
	this.textFieldE.htmlE.focus();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to set the focus to the textfield
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.setListBoxFocus = function() {
	this.listBoxE.htmlE.focus();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// sets Listbox selection
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.setSelection = function(index) {
	var options = this.listBoxE.htmlE.options;
	for (i = 0; i < options.length; i++) {
		if (i == index) {
			options[i].selected = true;
		} else {
			options[i].selected = false;
		}
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to update the textfield from selection
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.updateTextField = function() {
	var options = this.listBoxE.htmlE.options;
	for (i = 0; i < options.length; i++) {
		if (options[i].selected) {
			this.setText(options[i].value);
			this.saveState();
			break;
		}
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// called when the key is pressed up or down
// increments the list selection and shows the result in
// the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.incrementSelection = function(direction) {
	var options = this.listBoxE.htmlE.options;
	var selectedIndex = this.listBoxE.htmlE.selectedIndex;
	if (selectedIndex == -1) {
		// do a search for the best matching index
		selectedIndex = this.searchMatchingText();
		if (selectedIndex == -1)
			selectedIndex = 0;
	} else {
		if (direction > 0)
			selectedIndex = Math.min(options.length,selectedIndex+direction);
		else
			selectedIndex = Math.max(0,selectedIndex+direction);
	}
	this.setSelection(selectedIndex);
	this.updateTextField();
	this.setTextFieldFocus();
	this.selectText();
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Searchs for the index of the item that best matches the current
// text value.  Returns the index or -1 if none match
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.searchMatchingText = function() {
	var val = this.textFieldE.htmlE.value;
	if (val.length == 0)
		return -1;
	var options = this.listBoxE.htmlE.options;
	// the first search is for an exact match 
	for (i = 0; i < options.length; i++) {
		if (options[i].value == val)
			return i;
	}
	// and then failing that for a partial match 
	for (i = 0; i < options.length; i++) {
		if (options[i].value.indexOf(val) == 0)
			return i;
	}
	return -1;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Function called at list change time
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.onChange = function() {
	this.updateTextField();
	this.selectText();
	if (this.actionOnSelection) {
		E_setAction(this.htmlId, "selection");
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called at key press time in the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.onListKey  = function(event) {
	event = browser.isIE ? window.event : event;
	var key = event.keyCode ? event.keyCode : event.which;
	// ESC KEY
	if (key == 27 && this.listBoxE.isVisible()) { 
		this.hideListBox();
		this.setTextFieldFocus();
	// ENTER KEY
	} else if (key == 13 && this.listBoxE.isVisible()) { 
		this.hideListBox();
		this.updateTextField();
		this.setTextFieldFocus();
	}
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called at key press time in the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.onTextKey  = function(event) {
	event = browser.isIE ? window.event : event;
	var key = event.keyCode ? event.keyCode : event.which;

	// DOWN ARROW
	if (key == 40) { 		
		if (! this.listBoxE.isVisible()) {
			this.incrementSelection(+1);
		} else {
			this.setListBoxFocus();		
		}
	// UP ARROW
	} else if (key == 38) { 
		if (! this.listBoxE.isVisible()) {
			this.incrementSelection(-1);
		} else {
			this.setListBoxFocus();		
		}
	// ESC KEY
	} else if (key == 27) { 
		this.listBoxE.setVisible(false);

	// F4 KEY
	} else if (key == 27) { 
		this.toggle();

	// ENTER KEY
	} else if (key == 13) { 
		// save the state of things!
		if (this.actionOnEnter) {
			this.saveState();
		    E_setAction(this.htmlId, "enter");
		    //alert('State Saved');
			return;
		}
	} else {
		//
		// normal key strokes
		var matchesOccurred = this.rebuildOptions();
		if (this.textMatching && matchesOccurred) {
			this.adjustSize();
			this.showListBox();
		} else {
			this.hideListBox();
		}
	}
	this.saveState();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Rebuilds the array of "options" in the listbox,  returns True
// if matches occur
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.rebuildOptions = function(showAll) {
	var matchesOccurred = false;
	var textVal = this.textFieldE.htmlE.value;
	if (this.textMatching == false || showAll) {
		textVal = "";
	}
	var cb = this;
	var options = this.listBoxE.htmlE.options;
	// clear the current list of options
	options.length = 0; 
	// go through our internal options
   	for (i = 0; i < this.items.length; i++) {
		var val = this.items[i].itemValue;
	   	if (val.indexOf(textVal) == 0 || textVal.length == 0) {
	   		var option = document.createElement("OPTION");
			option.value = this.items[i].itemValue;
			option.text = this.items[i].itemValue;
			option.className = this.items[i].itemClass;
			options.add(option);
			matchesOccurred = true;
		}
   	}
   	return matchesOccurred;
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to toggle the display of the list
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.toggle = function() {
	if (this.listBoxE.isVisible()) {
		this.hideListBox();
		this.setTextFieldFocus();
	} else {
		this.rebuildOptions(true);
		var index = this.searchMatchingText();
		if (index >= 0) {
			this.setSelection(index);
			this.updateTextField();
			this.selectText();
		}
		this.showListBox();
		this.setListBoxFocus();
	}
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to set the text of the textfield
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.setText = function(textValue) {
	this.textFieldE.htmlE.value = textValue;
	//var delayedFunc = 'epg_EPComboBoxSetText("' + this.htmlId + '","'+ textValue + '")';
	//window.setTimeout(delayedFunc,10);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function called at set the text of the text field
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_EPComboBoxSetText(htmlId, textValue) {
	var combo = EPComboBoxMap[htmlId];
	if (combo != null)
		combo.textFieldE.htmlE.value = textValue;
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to set the text selection of the textfield
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.selectText = function() {
	var delayedFunc = 'epg_EPComboBoxSelectText("' + this.htmlId + '")';
	window.setTimeout(delayedFunc,10);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Delayed selectText function
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epg_EPComboBoxSelectText(htmlId) {
	var combo = EPComboBoxMap[htmlId];
	if (combo != null)
		combo.textFieldE.htmlE.select();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called at load time to init the EPComboBox
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.init = function(itemValue) {
	this.textFieldE = getElementById(this.htmlId);
	if (this.textFieldE == null)
		alert('ASSERT : EPComboBox init error : textfield not present');
		
	this.listBoxE = getElementById(this.htmlId+"List");
	if (this.listBoxE == null)
		alert('ASSERT : EPComboBox init error : list not present');

	this.toggleE = getElementById(this.htmlId+"Toggle");
	if (this.toggleE == null)
		alert('ASSERT : EPComboBox init error : toggle not present');

	// we manage the showing and hiding of the select field!
	epbe_registerExemptSelectField(this.htmlId+"List");

	this.rebuildOptions(true);
	this.adjustSize();
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Called to save the state of the component
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
EPComboBox.prototype.saveState = function() {
	E_setParameter(this.htmlId, this.textFieldE.htmlE.value);
}


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// Global function to find out an item and return it
// Can be used to inbvoke this object as in
// 		epcbObj('xyz').hide();
//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
function epcbOBJ(htmlId) {
	var combo = EPComboBoxMap[htmlId];
	if (combo == null)
		alert('ASSERT : EPComboBox error : object ' + htmlId + ' not found');
	return combo;
}




epbe_registerDocumentEventHandler("click",epEPComboBoxDocumentOnClickHandler,"comboxbox");
