/* 
 * This file is part of the Echo Point Project.  This project is a collection
 * of Components that have extended the Echo Web Application Framework.
 *
 * EchoPoint is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * EchoPoint is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Echo Point; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//========================================================================================================
// E - The element wrapper
//========================================================================================================

//- - - - - - - - - - - - - - - - - - - - 
// E - element object.  Represents
// an HTML element.  You must pass in a 
// valid HTML element.
//- - - - - - - - - - - - - - - - - - - - 
function E(htmlE) {
	this.htmlE = htmlE;
	this.id = null;
	if (htmlE.id)
		this.id = htmlE.id;
	else if (htmlE.name)
		this.id = htmlE.name;
	this.style = htmlE.style;
}

//- - - - - - - - - - - - - - - - - - - - 
// Returns a E object encapsulating the
// HTML element with the given Id or
// null if it can be found
//
function getElementById(htmlId) {
	var htmlE = _findElementImpl(htmlId, document);
	if (htmlE == null)
		return null;
	return new E(htmlE);
}

//- - - - - - - - - - - - - - - - - - - - 
// find implementation
function _findElementImpl(id, d) {
	var i, x;
	if (!d)
		d = document;
	var d = document;
	if (!(x = d[id]) && d.all)
		x = d.all[id];
	for (i = 0; !x && i < d.forms.length; i++)
		x = d.forms[i][id];
	for (i = 0; !x && d.layers && i < d.layers.length; i++)
		x = _findElementImpl(id, d.layers[i].document);
	if (!x && document.getElementById)
		x = document.getElementById(id);
	return x;
}

//- - - - - - - - - - - - - - - - - - - - 
// returns an array of E objects for
// the given htmlId
//
function getElementsById(htmlId) {
	var i, arr;
	if (browser.isDOM) {
		arr = document.getElementsByName(htmlId);
	} else if (browser.isIE) {
		arr = document.all[htmlId];
	} else if (browser.isNS) {
		arr = document.layers[htmlId];
	}
	var eArray = new Array();
	if (arr == null)
		return eArray;

	for (i = 0; i < arr.length; i++) {
		eArray[i] = new E(arr[i]);
	}
	return eArray;
}

//- - - - - - - - - - - - - - - - - - - - 
// getParent - may return null
//
E.prototype.getParent = function() {
	if (this.htmlE.parentNode)
		return new E(this.htmlE.parentNode);
	if (this.htmlE.parentElement)
		return new E(this.htmlE.parentElement);
	return null;
}

//- - - - - - - - - - - - - - - - - - - - 
// isAncestorOf - may return null
//
E.prototype.isAncestorOf = function(e) {
	if (e) {
		if (this.htmlE == e.htmlE)
			return true;
	
		var parentE = this.getParent();
		while(parentE) {
			if (parentE.htmlE == e.htmlE)
				return true;
			parentE = parentE.getParent();
		}
	}
	return false;
}


//- - - - - - - - - - - - - - - - - - - - 
// getTag
//
E.prototype.getTag = function() {
	if (this.htmlE.tagName)
		return this.htmlE.tagName
	return "unknown";
}

//- - - - - - - - - - - - - - - - - - - - 
// get and set X 
//
E.prototype.getX = function() {
	var x = 0;
	if (browser.isNS)
		x = this.htmlE.left;
	else if (browser.isOpera)
		x = this.htmlE.style.pixelLeft;
	else 
		x = this.htmlE.offsetLeft;
	return x;
}

E.prototype.setX = function(x) {
	(browser.isIE || browser.isDOM) ? this.htmlE.style.left = x : (browser.isNS) ? this.htmlE.left = x : this.htmlE.style.pixelLeft = x;
}

//- - - - - - - - - - - - - - - - - - - - 
// get and set Y
//
E.prototype.getY = function() {
	var y = 0;
	if (browser.isNS)
		y = this.htmlE.top;
	else if (browser.isOpera)
		y = this.htmlE.style.pixelTop;
	else 
		y = this.htmlE.offsetTop;
	return y;
}

E.prototype.setY = function(y) {
	(browser.isIE || browser.isDOM) ? this.htmlE.style.top = y : (browser.isNS) ? this.htmlE.top = y : this.htmlE.style.pixelTop = y;
}

//- - - - - - - - - - - - - - - - - - - - 
// get and set Z
//

E.prototype.getZ = function() {
	var z = 0;
	if (this.htmlE.zIndex)
		z = this.htmlE.zIndex;
	else if (this.htmlE.style)
		z = parseInt(this.htmlE.style.zIndex, 10);
	if (isNaN(z))
		z = 0;
	return z;
}

E.prototype.setZ = function(z) {
	if (this.htmlE.zIndex)
		this.htmlE.zIndex = z;
	else {
		if (this.htmlE.style)
			this.htmlE.style.zIndex = z;
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// get and set Height
//

E.prototype.getHeight = function() {
	var h = 0;
	if (browser.isNS) {
		h = (this.htmlE.height) ? this.htmlE.height : this.htmlE.clip.height;
		return h;
	}
	h = (browser.isOpera) ? this.htmlE.style.pixelHeight : this.htmlE.offsetHeight;
	return h;
}

E.prototype.setHeight = function(h) {
	if (browser.isNS) {
		if (this.htmlE.clip)
			this.htmlE.clip.bottom = h;
	} else if (browser.isOpera)
		this.htmlE.style.pixelHeight = h;
	else
		this.htmlE.style.height = h;
}

//- - - - - - - - - - - - - - - - - - - - 
// get and set Width
//

E.prototype.getWidth = function() {
	var w = 0;
	if (browser.isNS) {
		w = (this.htmlE.width) ? this.htmlE.width : this.htmlE.clip.width;
		return w;
	}
	w = (browser.isOpera) ? this.htmlE.style.pixelWidth : this.htmlE.offsetWidth;
	return w;
}

E.prototype.setWidth = function(w) {
	if (browser.isNS) {
		if (this.htmlE.clip)
			this.htmlE.clip.right = w;
	} else if (browser.isOpera)
		this.htmlE.style.pixelWidth = w;
	else
		this.htmlE.style.width = w;
}

//- - - - - - - - - - - - - - - - - - - - 
// get page X
//

E.prototype.getPageX = function() {
	if (browser.isNS) {
		var x = (this.htmlE.pageX) ? this.htmlE.pageX : this.htmlE.x;
		return x;
	} else if (browser.isOpera) {
		var x = 0;
		var elem = this.htmlE;
		while (eval(elem)) {
			x += elem.style.pixelLeft;
			elem = elem.offsetParent;
		}
		return x;
	} else {
		var x = 0;
		var elem = this.htmlE;
		while (eval(elem)) {
			x += elem.offsetLeft;
			elem = elem.offsetParent;
			//if (elem && elem.scrollLeft)
			//	x -= elem.scrollLeft;
		}
		if (browser.isIE)
			x += document.body.clientLeft;
		return x;
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// get page Y
//

E.prototype.getPageY = function() {
	if (browser.isNS) {
		var y = (this.htmlE.pageY) ? this.htmlE.pageY : this.htmlE.y;
		return y;
	} else if (browser.isOpera) {
		var y = 0;
		var elem = this.htmlE;
		while (eval(elem)) {
			y += elem.style.pixelTop;
			elem = elem.offsetParent;
		}
		return y;
	} else {
		var y = 0;
		var elem = this.htmlE;
		while (eval(elem)) {
			y += elem.offsetTop;
			elem = elem.offsetParent;
			//if (eval(elem) && eval(elem.scrollTop)) {
			//	y -= elem.scrollTop;
			//}	
		}
		if (browser.isIE)
			y += document.body.clientTop;
		return y;
	}
}

//- - - - - - - - - - - - - - - - - - - - 
// get scrollX
E.prototype.getScrollX = function() {
	if (this.htmlE.scrollLeft)
		return this.htmlE.scrollLeft;
	return 0;
}

//- - - - - - - - - - - - - - - - - - - - 
// get scrollY
E.prototype.getScrollY = function() {
	if (this.htmlE.scrollTop)
		return this.htmlE.scrollTop;
	return 0;
}

//- - - - - - - - - - - - - - - - - - - - 
// calculates all scrollX values from the 
// current element upwards
//
E.prototype.calcScrollX = function() {
	var e = this;
	var scroll = 0;
	while (e.getParent()) {
		e = e.getParent();
		if (e.getTag() == "BODY")
			break;
		scroll += e.getScrollX();
	}
	return scroll;
}

//- - - - - - - - - - - - - - - - - - - - 
// calculates all scrollY values from the 
// current element upwards
//
E.prototype.calcScrollY = function() {
	var e = this;
	var scroll = 0;
	while (e.getParent()) {
		e = e.getParent();
		if (e.getTag() == "BODY")
			break;
		scroll += e.getScrollY();
	}
	return scroll;
}



//- - - - - - - - - - - - - - - - - - - - 
// get page Width

E.prototype.getPageWidth = function() {
	if (browser.NS || browser.Opera)
		return frame.innerWidth;
	else
		return document.body.clientWidth;
}

//- - - - - - - - - - - - - - - - - - - - 
// get page Height
//

E.prototype.getPageHeight = function() {
	if (browser.NS || browser.Opera)
		return frame.innerHeight;
	else
		return document.body.clientHeight;
}

//- - - - - - - - - - - - - - - - - - - - 
// get a style attribute value
//
E.prototype.getStyle = function(styleAttr) {
	if (browser.isIE5 || browser.isDOM)
		return eval("this.htmlE.style." + styleAttr);
}

//- - - - - - - - - - - - - - - - - - - - 
// sets a style attribute to value
//
E.prototype.setStyle = function(styleAttr, styleValue) {
	if (browser.isIE5 || browser.isDOM)
		eval("this.htmlE.style." + styleAttr + " = '" + styleValue + "'");
}

//- - - - - - - - - - - - - - - - - - - - 
// moveTo {x,y}
//
E.prototype.moveTo = function(x, y) {
	this.setX(x);
	this.setY(y);
}

//- - - - - - - - - - - - - - - - - - - - 
// moveBy {deltax, deltay}
//
E.prototype.moveBy = function(deltaX, deltaY) {
	this.setX(this.getX() + deltaX);
	this.setY(this.getY() + deltaY);
}

//- - - - - - - - - - - - - - - - - - - - 
// Write HTML to the element
//
E.prototype.writeHTML = function(htmlText) {
	if (this.htmlE.innerHTML)
		this.htmlE.innerHTML = htmlText;
}

//--------------------------------------------
// Returns true if the testE inclusively overlaps 
// with the current element with an optional 
// toleranceFactor
//--------------------------------------------
E.prototype.overlaps = function(testE, toleranceFactor) {
	epbe_overlaps(this,testE,toleranceFactor);
}

//- - - - - - - - - - - - - - - - - - - - 
// isVisible
//
E.prototype.isVisible = function() {
	if (this.htmlE.style)
		if (this.htmlE.style.visibility == 'hidden')
			return false;
	return true;
}

//- - - - - - - - - - - - - - - - - - - - 
// set Visible
//
E.prototype.setVisible = function(isVisible) {
	if (isVisible)
		this.setStyle('visibility', 'visible');
	else
		this.setStyle('visibility', 'hidden');
}

//- - - - - - - - - - - - - - - - - - - - 
// isDisplayed
//
E.prototype.isDisplayed = function() {
	if (this.htmlE.style)
		if (this.htmlE.style.display == 'none')
			return false;
	return true;
}

//- - - - - - - - - - - - - - - - - - - - 
// set Displayed
//
E.prototype.setDisplayed = function(isDisplayed) {
	if (isDisplayed)
		this.setStyle('display', '');
	else
		this.setStyle('display', 'none');
}

//- - - - - - - - - - - - - - - - - - - - 
// isAbsolute
//
E.prototype.isAbsolute = function() {
	if (this.htmlE.style)
		if (this.htmlE.style.position == 'absolute')
			return true;
	return false;
}

//- - - - - - - - - - - - - - - - - - - - 
// isRelative
//
E.prototype.isRelative = function() {
	if (this.htmlE.style)
		if (this.htmlE.style.position == 'relative')
			return true;
	return false;
}


//- - - - - - - - - - - - - - - - - - - - 
// add and event listener function
//
E.prototype.addEvent = function(e, f, c) {
	if (this.htmlE.addEventListener)
		this.htmlE.addEventListener(e, f, c);
	else if (this.htmlE.attachEvent)
		this.htmlE.attachEvent("on" + e, f);
	else
		eval("this.htmlE.on" + e + "=" + f);
}

//- - - - - - - - - - - - - - - - - - - - 
// report settings for a E element
//
E.prototype.report = function() {
	var s = "";
	if (this.htmlE.id)
		s += " id:" + this.htmlE.id;
	if (this.htmlE.name)
		s += " name:" + this.htmlE.name;

	s += " x:" + this.getX();
	s += " y:" + this.getY();
	s += " z:" + this.getZ();
	s += " w:" + this.getWidth();
	s += " h:" + this.getHeight();
	s += " px:" + this.getPageX();
	s += " py:" + this.getPageY();
	s += " scrlX:" + this.getScrollX();
	s += " scrlY:" + this.getScrollY();
	if (this.htmlE.style) {
		s += " visibility:" + this.htmlE.style.visibility;
		s += " display:" + this.htmlE.style.display;
		s += " position:" + this.htmlE.style.position;
	}
	return s;
}

//- - - - - - - - - - - - - - - - - - - - 
// function to convert integers to hex 
// strings.
//- - - - - - - - - - - - - - - - - - - - 
var _hexDigitsArray = new Array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');

function _toHex(i) {
	var a2 = '';
	var ihex = Math.floor(eval(i + '/16'));
	var idiff = eval(i + '-('+ihex + '*16)');
	a2 = _hexDigitsArray[idiff] + a2;
	while (ihex >= 16) {
		itmp = Math.floor(eval(ihex + '/16'));
		idiff = eval(ihex + '-('+itmp + '*16)');
		a2 = _hexDigitsArray[idiff] + a2;
		ihex = itmp;
	}
	var a1 = _hexDigitsArray[ihex];
	return a1 + a2;
}

//- - - - - - - - - - - - - - - - - - - - 
// Normalises a color string into the
// rgb(rr,gg,bb) form.  
//- - - - - - - - - - - - - - - - - - - - 
function epbe_normalizeColorRGB(clrStr) {
	var index = clrStr.indexOf('rgb(');
	if (index == 0)
		return clrStr;
	index = clrStr.indexOf('#');
	if (index == 0) {
		var r = parseInt(clrStr.substring(1, 3), 16);
		var g = parseInt(clrStr.substring(3, 5), 16);
		var b = parseInt(clrStr.substring(5, 7), 16);

		return 'rgb('+r + ',' + g + ',' + b + ')';
	}
	alert('ASSERT : invalid color format : ' +clrStr);
}

//- - - - - - - - - - - - - - - - - - - - 
// Normalises a color string into the
// #rrggbb form.
//- - - - - - - - - - - - - - - - - - - - 
function epbe_normalizeColorHex(clrStr) {
	var index1 = clrStr.indexOf('#');
	var index2 = index1;
	if (index1 == 0)
		return clrStr;
	index1 = clrStr.indexOf('rgb(');
	index2 = clrStr.indexOf(')');
	if (index1 == 0) {
		var digits = clrStr.substring(index1 + 4, index2);
		var clrs = digits.split(',');
		var r = parseInt(clrs[0]);
		var g = parseInt(clrs[1]);
		var b = parseInt(clrs[2]);

		return '#' + _toHex(r) + _toHex(g) + _toHex(b);
	}
	alert('ASSERT : invalid color format : ' +clrStr);
}



//------------------------------------------------------------
// Adds an event type to the object's single event handler
// called 'eventHandler' for the given element
//
// The object must have an event handler function in the form :
//
//		obj.eventHandler(event)
//------------------------------------------------------------
function epbe_AddEventHandler(obj, elem, win, eventType) {
	var f = function(event) {
		obj.eventHandler(browser.isIE ? win.event : event);
	}
	if (elem.addEventListener) // MOZ
		elem.addEventListener(eventType, f, true);
	else if (elem.attachEvent) // IE
		elem.attachEvent("on" + eventType, f);
	else
		eval("elem.on" + eventType + "=" + f);
}


//------------------------------------------------------------
// Called to cancel an event from bubbling up the chain
//------------------------------------------------------------
function epbe_CancelEvent(ev) {
	if (browser.isIE) {
		ev.cancelBubble = true;
		ev.returnValue = false;
	}
	if (browser.isNS6) {
		ev.preventDefault();
		ev.stopPropagation();
	}
}